
local util = {}

local lfs = require("lfs")

function util.write_file(name, data)
   local fd = io.open(name, "w")
   if not fd then
      return nil, "could not write "..name
   end
   fd:write(data)
   fd:close()
   return true
end

function util.read_file(name, offset)
   offset = offset or 0
   local fd, err = io.open(name, "r")
   if not fd then
      return nil, "could not read "..name.." : "..err
   end
   local n_bytes, err = fd:seek("set", offset)
   if not n_bytes then
      return nil, "could not read "..name.." : "..err
   end
   local data = fd:read("*a")
   fd:close()
   return data
end

function util.debug_table(tbl, lvl)
   lvl = lvl or 0
   local indent = ("   "):rep(lvl)
   local out = {"{\n"}
   for k,v in pairs(tbl) do
      table.insert(out, indent.."   ")
      table.insert(out, tostring(k).." = ")
      table.insert(out, (type(v) == "table" and util.debug_table(v, lvl + 1) or tostring(v)))
      table.insert(out, "\n")
   end
   table.insert(out, indent.."}")
   return table.concat(out)
end

local function can_open(path, mode)
   local fd = io.open(path, mode)
   return fd and fd:close() or false
end

function util.list_local_path(path_name)
   local attrs = lfs.attributes(path_name)
   local list = {}
   if attrs and attrs.mode == "directory" then
      for file_name in lfs.dir(path_name) do
	 if file_name ~= "." and file_name ~= ".." then
	    local file_path
	    if path_name == "/" then
	       file_path = "/"..file_name
	    else
	       file_path = path_name.."/"..file_name
	    end		   
	    print(file_path)
            list[#list + 1] = util.check_local_path(file_path)
	    print(list[#list-1])
         end
      end
      return list
   end	   
   return nil, "Could not list path"
end

function util.check_local_path(path_name)
   local attrs = lfs.symlinkattributes(path_name)
   print(util.debug_table(attrs))
   if attrs then
      return {
         path = path_name,
         size_bytes = attrs.size,
         is_dir = attrs.mode == "directory",
         is_symlink = attrs.mode == "link",
         is_readable = can_open(path_name, "r"),
         is_writable = can_open(path_name, "w"),
         is_executable = true, --FIXME not obtainable using lfs
         link_path = path_name, --FIXME not obtainable using lfs
      }
   end
   return nil, "Could not read path"
end

function util.convert_to_list(map, key)
   local i = 1
   local tab = {}
   while (map[key.."."..i]) do
      table.insert(tab, map[key.."."..i])
      i = i + 1
  end
  return tab
end

function util.removedir(path)
    for file in lfs.dir(path) do
        if file ~= "." and file ~= ".." then
            local f = path..'/'..file
            local attr = lfs.attributes(f)
            assert(attr)
            if attr.mode == "directory" then
                util.removedir(f)
            else
                local ok, err = os.remove(f)
                if not ok then
                    return ok, err
                end
            end
        end
    end
    return lfs.rmdir(path)
end

return util
